@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007-2020 by the individuals mentioned in the source code history
+ *  Copyright 2007-2021 by the individuals mentioned in the source code history
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -39,7 +39,15 @@
 #include "omxState.h"
 #include "omxExpectation.h"
 
-struct omxFitFunction {
+class omxFitFunction {
+ private:
+  bool applyPenalty;
+  std::vector< Penalty* > penalties;
+protected:
+ public:
+  void subCompute(int want, FitContext *fc); // used by multigroup, hidden markov
+  void recompute(int want, FitContext *fc) { subCompute(want, fc); } // used by omxRecompute
+
 	SEXP rObj;
 	omxExpectation* expectation;
 
@@ -48,6 +56,7 @@ struct omxFitFunction {
 	const char* fitType;
 
 	omxMatrix* matrix;
+  double scale;
 	bool initialized;
 	bool hessianAvailable;
 	FitStatisticUnits units;
@@ -59,15 +68,16 @@ struct omxFitFunction {
 	std::vector<int> gradMap;
 	std::vector<int> missingGrad;
 
-	omxFitFunction() : rObj(0), expectation(0), initialized(false),
+	omxFitFunction() : rObj(0), expectation(0), scale(1), initialized(false),
 		hessianAvailable(false), units(FIT_UNITS_UNINITIALIZED), canDuplicate(false),
     openmpUser(false), verbose(0), derivCount(0) {};
 	virtual ~omxFitFunction() {};
 	virtual omxFitFunction *initMorph();
 	virtual void init()=0;
-	virtual void compute(int ffcompute, FitContext *fc)=0;
+	void compute(int ffcompute, FitContext *fc);
+	virtual void compute2(int ffcompute, FitContext *fc)=0;
 	virtual void invalidateCache() {};
-	virtual void traverse(std::function<void(omxMatrix*)> &fn);
+	virtual void traverse(std::function<void(omxMatrix*)> fn);
 
 	// addOutput should only be used for returning global results
 	virtual void addOutput(MxRList *out) {};
@@ -79,6 +89,8 @@ struct omxFitFunction {
   void invalidateGradient(FitContext *fc);
 	void setUnitsFromName(const char *name);
 	const char *name() const { return matrix->name(); }
+  void connectPenalties();
+	virtual void sufficientDerivs2Grad_NormalContinuous() {};
 };
 
 /* Initialize and Destroy */
@@ -91,7 +103,6 @@ void omxCompleteFitFunction(omxMatrix *om);
 	void omxGetFitFunctionStandardErrors(omxFitFunction *oo);					// Get Standard Errors
 
 /* FitFunction-specific implementations of matrix functions */
-void omxFitFunctionCompute(omxFitFunction *off, int want, FitContext *fc);  // deprecated, use ComputeFit
 	void omxDuplicateFitMatrix(omxMatrix *tgt, const omxMatrix *src, omxState* targetState);
 
 omxMatrix* omxNewMatrixFromSlot(SEXP rObj, omxState* state, const char* slotName);
@@ -116,8 +127,6 @@ void ba81SetFreeVarGroup(omxFitFunction *oo, FreeVarGroup *fvg);
 void ComputeFit(const char *callerName, omxMatrix *fitMat, int want, FitContext *fc);
 void loglikelihoodCIFun(omxFitFunction* oo, int ffcompute, FitContext *fc);
 
-double totalLogLikelihood(omxMatrix *fitMat); // deprecated, use ComputeFit
-
 const char *fitUnitsToName(FitStatisticUnits units);
 bool fitUnitsIsChiSq(FitStatisticUnits units);
 SEXP makeFitUnitsFactor(SEXP obj);
