@@ -597,9 +597,52 @@ double trace_prod(const Eigen::MatrixBase<T1> &t1, const Eigen::MatrixBase<T2> &
 	return sum;
 }
 
+//For the trace of the product of two symmetric matrices:
+template <typename T1, typename T2>
+double trace_prod_symm(const Eigen::MatrixBase<T1> &t1, const Eigen::MatrixBase<T2> &t2)
+{
+	int r=0, c=0;
+	double sum = 0.0;
+	for(c=0; c < t1.rows(); c++){
+		for(r=c; r < t1.rows(); r++){
+			sum += (r==c) ? t1(r,c)*t2(r,c) : 2*t1(r,c)*t2(r,c);
+		}
+	}
+	return sum;
+}
+
 void MoorePenroseInverseSq(Eigen::Ref<Eigen::MatrixXd> mat);
 void MoorePenroseInverse(Eigen::Ref<Eigen::MatrixXd> in, Eigen::Ref<Eigen::MatrixXd> out);
 
+template <typename T1>
+void subtractFromIdentityMatrixInPlace(Eigen::MatrixBase<T1> &M)
+{
+	M *= -1.0;
+	M.diagonal().array() += 1.0;
+}
+
+
+//N.B. this function assumes that the same row and column indices are being filtered, i.e. if row i is to be filtered, then so is column i:
+template <typename T1, typename T2>
+void copyAndFilterCases(Eigen::MatrixBase<T1> &inMat, Eigen::MatrixBase<T2> &outMat, Eigen::VectorXi &todrop, bool symmetric, int origDim)
+{
+	//Possible TODO : is column- versus row-major order relevant to this function's performance?
+	int nextCol = 0;
+	int nextRow = 0;
+	
+	for(int j = 0; j < origDim; j++){ //<--j indexes columns
+		if(todrop[j]) continue;
+		nextRow = (symmetric ? nextCol : 0);
+		for(int k = (symmetric ? j : 0); k < origDim; k++){ //<--k indexes rows
+			if(todrop[k]) continue;
+			outMat(nextRow,nextCol) = inMat(k,j);
+			nextRow++;
+		}
+		nextCol++;
+	}
+}
+
+
 // https://forum.kde.org/viewtopic.php?f=74&t=96706
 // https://forum.kde.org/viewtopic.php?f=74&t=124421
 // https://forum.kde.org/viewtopic.php?f=74&t=91271
